/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.util;

import com.google.common.base.Joiner;

import java.io.File;

import org.apache.log4j.Logger;

import static com.google.common.base.Strings.*;
import static com.google.common.collect.ObjectArrays.*;

public abstract class OperatingSystem {
    public static final String name = System.getProperty("os.name");
    public static final boolean isWindows = name.startsWith("Windows");
    public static final boolean isUnix = !isWindows;
    public static final Unix unix = new Unix();
    public static final Windows windows = new Windows();
    public static final OperatingSystem operatingSystem = isWindows? windows : unix;
    public static final String javaLibraryPathPropertyName = "java.library.path";
    private static final Logger log = Logger.getLogger(OperatingSystem.class);
    
    public final String makeModuleFilename(String path) {
        File file = new File(path);
        return new File(file.getParent(), System.mapLibraryName(file.getName())).getPath();
    }
    
    public abstract String makeExecutableFilename(String name);
    public abstract String getDynamicLibraryEnvironmentVariableName();
//    protected abstract void removeAbsoluteDirectoryRecursivly(File directory);
//
//    public void removeDirectoryRecursivly(File directory) {
//        try {
//            File dir = directory.getCanonicalFile();
//            checkArgument(dir.isDirectory(), "Not a directory: %s", directory);
//            removeAbsoluteDirectoryRecursivly(dir);
//            if (directory.exists())  throw new RuntimeException("Removal of directory failed: " + directory);
//        } catch (IOException x) { throw new RuntimeException(x); }
//    }
//
//    public void executeShell(File program, String... arguments) {
//        try {
//            checkArgument(program.isFile(), "Not a file: %s", program);
//            String[] commandAndArguments = ObjectArrays.concat(new String[]{program.toString()}, arguments, String.class);
//            if (log.isDebugEnabled()) log.debug("exec " + Joiner.on(' ').join(commandAndArguments));
//            Process process = Runtime.getRuntime().exec(commandAndArguments);
//            *** Hier muss vermutlich process.getOutputStream() und .getErrorStream() ausgelesen werden, um Blockade zu vermeiden ***
//            process.waitFor();
//            if (process.exitValue() != 0)
//                throw new RuntimeException("Execution of program failed with exit code " + process.exitValue() + ": " + program);
//        } catch (IOException x) { throw new RuntimeException(x); }
//        catch (InterruptedException x) { throw new RuntimeException(x); }
//    }
    

    public static class Windows extends OperatingSystem {
        @Override public final String makeExecutableFilename(String name) {
            return name + ".exe";
        }

        @Override public final String getDynamicLibraryEnvironmentVariableName() {
            return "PATH";
        }

//        @Override public void removeAbsoluteDirectoryRecursivly(File directory) {
//            executeShell(commandFile(), "rd", "/s", directory.toString());
//        }
//
//        private File commandFile() {
//            return new File(windowsDirectory() + File.separator + "system32", "cmd.exe");
//        }
//
//        private File windowsDirectory() {
//            String windir = System.getenv("WINDIR");
//            if (isNullOrEmpty(windir))  throw new RuntimeException("Environment variable WINDIR is missing");
//            File result = new File(windir);
//            assert result.isDirectory();
//            return result;
//        }
    }
    

    public static class Unix extends OperatingSystem {
        @Override public final String makeExecutableFilename(String name) {
            return name;
        }

        @Override public final String getDynamicLibraryEnvironmentVariableName() {
            return "LD_LIBRARY_PATH";
        }

//        @Override public void removeAbsoluteDirectoryRecursivly(File directory) {
//            executeShell(new File("/bin/rm"), "-rf", directory.toString());
//        }
    }

    public static void prependJavaLibraryPath(File f) {
        String a = System.getProperty(javaLibraryPathPropertyName);
        String c = concatFileAndPathChain(f, nullToEmpty(a));
        if (!c.equals(a)) {
            System.setProperty(javaLibraryPathPropertyName, c);
            log.debug("Property " + javaLibraryPathPropertyName + "=" + c);
        }
    }

    public static String concatFileAndPathChain(File f, String pathChain) {
        String abs = f.getAbsolutePath();
        String[] b = pathChain.split(File.pathSeparator);
        for (int i = 0; i < b.length; i++)  if (b[i].isEmpty() || b[i].equals(abs))  b[i] = null;
        return Joiner.on(File.pathSeparatorChar).skipNulls().join(concat(abs, b));
    }
}
