/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.scheduler.job;

import static com.sos.scheduler.messages.JSMessages.JSJ_D_0010;
import static com.sos.scheduler.messages.JSMessages.JSJ_D_0032;
import static com.sos.scheduler.messages.JSMessages.JSJ_D_0040;
import static com.sos.scheduler.messages.JSMessages.JSJ_D_0070;
import static com.sos.scheduler.messages.JSMessages.JSJ_D_0080;
import static com.sos.scheduler.messages.JSMessages.JSJ_E_0009;
import static com.sos.scheduler.messages.JSMessages.JSJ_F_0010;
import static com.sos.scheduler.messages.JSMessages.JSJ_F_0050;
import static com.sos.scheduler.messages.JSMessages.JSJ_F_0060;
import static com.sos.scheduler.messages.JSMessages.JSJ_I_0010;
import static com.sos.scheduler.messages.JSMessages.JSJ_I_0020;
import static com.sos.scheduler.messages.JSMessages.JSJ_I_0030;
import static com.sos.scheduler.messages.JSMessages.LOG_I_0010;
import static com.sos.scheduler.messages.JSMessages.LOG_I_0020;

import java.io.File;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.regex.Pattern;

import org.apache.log4j.Appender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;

import sos.scheduler.interfaces.IJobSchedulerMonitor_impl;
import sos.spooler.Job;
import sos.spooler.Order;
import sos.spooler.Supervisor_client;
import sos.spooler.Variable_set;

import com.sos.JSHelper.Basics.JSJobUtilities;
import com.sos.JSHelper.Basics.JSVersionInfo;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Logging.JobSchedulerLog4JAppender;
import com.sos.JSHelper.Logging.Log4JHelper;
import com.sos.JSHelper.Options.JSOptionsClass;
import com.sos.i18n.annotation.I18NResourceBundle;
import com.sos.localization.Messages;
import com.sos.localization.SOSMsg;

/**
* \file JobSchedulerJobAdapter.java
*
* \class JobSchedulerJobAdapter
*
* \brief
*  JobSchedulerJobAdapter is the base-class for a Java-API Adapter-Class.
*
* \details
*
* \code
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author KB
* @version $Id: JobSchedulerJobAdapter.java 18607 2012-12-06 14:41:46Z ur $13.06.2010
* \see reference
*
* Created on 13.06.2010 11:34:05
*
*
 */
@I18NResourceBundle(baseName = "com_sos_scheduler_messages", defaultLocale = "en")
public class JobSchedulerJobAdapter extends JobSchedulerJob implements JSJobUtilities, IJSCommands, IJobSchedulerMonitor_impl {
	@SuppressWarnings("unused")
	private final String				conSVNVersion		= "$Id: JobSchedulerJobAdapter.java 18607 2012-12-06 14:41:46Z ur $";

	private final String				conClassName		= "JobSchedulerJobAdapter";
	public final String					conMessageFilePath	= "com_sos_scheduler_messages";
	protected Variable_set				objJobOrOrderParams	= null;
	protected static Logger				logger				= Logger.getLogger(JobSchedulerJobAdapter.class);
	@SuppressWarnings("unused")
	private static Log4JHelper			objLogger			= null;
	protected Messages					Messages			= null;
	private JobSchedulerLog4JAppender	objJSAppender		= null;

	protected HashMap<String, String>	SchedulerParameters	= new HashMap<String, String>();
	protected HashMap<String, String>	hsmParameters		= null;
	protected final String				EMPTY_STRING		= "";

	public JobSchedulerJobAdapter() {
		Messages = new Messages(conMessageFilePath, Locale.getDefault());
		SOSMsg.flgShowFullMessageText = true;
	}

	@Override
	public boolean spooler_init() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::spooler_init";
		Messages = new Messages(conMessageFilePath, Locale.getDefault());
		return super.spooler_init();
	}

	@Override
	public boolean spooler_process() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::spooler_process";
		try {
			super.spooler_process();
			initializeLog4jAppenderClass();
			logger.info(JSVersionInfo.getVersionString());
		}
		catch (JobSchedulerException e) {
			return signalFailure();
		}
		catch (RuntimeException e) {
			return signalFailure();
		}
		catch (Exception e) {
			return signalFailure();
		}
		finally {
		} // finally
			// return (spooler_task.job().order_queue() != null);
		return signalSuccess();
	} // spooler_process

	protected void initializeLog4jAppenderClass() {
		String strJobName = this.getJobName();
		if (strJobName == null) {
			if (spooler_job != null) {
				this.setJobName(spooler_job.name());
			}
			strJobName = this.getJobName();
		}
		strJobName = strJobName.replace('/', '-');
		Log4JHelper.flgUseJobSchedulerLog4JAppender = true;
		File fleLog4JFile = null;
		JSOptionsClass objOC = new JSOptionsClass();
		if (objOC.log4jPropertyFileName.isDefault() == false) {
			fleLog4JFile = objOC.log4jPropertyFileName.JSFile();
		}
		else {
			fleLog4JFile = new File("./" + strJobName + "-log4j.properties");
			if (fleLog4JFile.exists() == false) {
				fleLog4JFile = new File("./" + "log4j.properties");
			}
		}
		objLogger = new Log4JHelper(fleLog4JFile.getAbsolutePath());
		logger = Logger.getRootLogger();
		/**
		 * the JobSchedulerLog4JAppender is used as the stdout-appender
		 * Therefore the code-snippet below asked log4j what the stdout-appender
		 * is and if it is the JobSchedulerLog4JAppender, the Instance of the
		 * sosJobSchedulerLogger is set.
		 *
		 * All Log4J-Messages are redirected to the sosLogger.
		 */
		Appender objStdoutAppender = logger.getAppender("stdout"); //$NON-NLS-1$
		if (objStdoutAppender instanceof JobSchedulerLog4JAppender) {
			objJSAppender = (JobSchedulerLog4JAppender) objStdoutAppender;
			objJSAppender.setSchedulerLogger(sosLogger);
			LOG_I_0020.toLog();
			if (spooler_log.level() < 0) {
				logger.setLevel(Level.DEBUG);
			}
		}
		if (isNull(objJSAppender)) {
			SimpleLayout layout = new SimpleLayout();
			objJSAppender = new JobSchedulerLog4JAppender(layout);
			Appender consoleAppender = objJSAppender; // JobSchedulerLog4JAppender(layout);
			logger.addAppender(consoleAppender);
			// ALL | DEBUG | INFO | WARN | ERROR | FATAL | OFF:
			logger.setLevel(Level.INFO);
			LOG_I_0010.toLog();
		}
		objJSAppender.setSchedulerLogger(sosLogger);
	}

	/**
	 *
	 * \brief getSchedulerParameterAsProperties
	 *
	 * \details
	 *
	 * \return HashMap<String,String>
	 *
	 * @param pSchedulerParameterSet
	 * @return
	 * @throws Exception
	 */
	protected HashMap<String, String> getSchedulerParameterAsProperties(final Variable_set pSchedulerParameterSet) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getSchedulerParameterAsProperties";
		SchedulerParameters = new HashMap<String, String>();
		try {
			if (isNotNull(pSchedulerParameterSet)) {
				SchedulerParameters = convertVariableSet2HashMap(pSchedulerParameterSet);
				for (String key : SchedulerParameters.keySet()) {
					String value = SchedulerParameters.get(key);
					String replacedValue = replaceVars(SchedulerParameters, key, value);
					if (replacedValue.equalsIgnoreCase(value) == false) {
						SchedulerParameters.put(key, replacedValue);
					}
				}
			}
			SchedulerParameters = DeleteCurrentNodeNameFromKeys(SchedulerParameters);
		}
		catch (Exception e) {
			throw new JobSchedulerException(JSJ_F_0060.params(StackTrace2String(e)), e);
		}
		return SchedulerParameters;
	}

	protected HashMap<String, String> convertVariableSet2HashMap(final Variable_set variableSet) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::convertVariableSet2HashMap";
		// System.out.println(conClassName);
		HashMap<String, String> result = new HashMap<String, String>();
		try {
			if (isNotNull(variableSet)) {
				String[] names = variableSet.names().split(";");
				String value = EMPTY_STRING;
				for (String key : names) {
					value = EMPTY_STRING;
					/**
					 * the variable_set is able to handle the value of a variable as an
					 * Object. In Java this class (the HashMap) is defined as <String,String> but it is possible
					 * e.g. in JavaScript to set a value as any object.
					 *
					 * Values with other types than string and integer are ignored, e.g. the
					 * value of this parameter is set to space.
					 */
					Object objO = variableSet.var(key);
					if (objO instanceof String) {
						value = variableSet.var(key);
					}
					else {
						if (objO instanceof Integer) {
							Integer intI = (Integer) objO;
							value = intI.toString();
						}
					}

					result.put(key, value);
					result.put(key.replaceAll("_", EMPTY_STRING).toLowerCase(), value);
				}

			}

			return result;
		}
		catch (Exception e) {
			throw new JobSchedulerException(JSJ_F_0060.params(StackTrace2String(e)), e);
		}
	}

	public HashMap<String, String> DeleteCurrentNodeNameFromKeys(final HashMap<String, String> pSchedulerParameterSet) {
		/**
		 * Delete the NodeName (StepName) for all parameters dedicated to the current step
		 * The name of the current step is the return-value of "getCurrentNodeName".
		 */
		String strCurrentNodeName = getCurrentNodeName() + "/";
		int intNNLen = strCurrentNodeName.length();
		HashMap<String, String> newSchedulerParameters = new HashMap<String, String>();
		newSchedulerParameters.putAll(pSchedulerParameterSet);
		Set<Map.Entry<String, String>> set = pSchedulerParameterSet.entrySet();
		for (Map.Entry<String, String> entry : set) {
			String key = entry.getKey();
			if (key.startsWith(strCurrentNodeName) == true) {
				String val = entry.getValue();
				key = key.substring(intNNLen);
				newSchedulerParameters.put(key, val);
			}
		}

		return newSchedulerParameters;
	}

	protected HashMap<String, String> getSchedulerParameterAsProperties(final HashMap<String, String> pSchedulerParameterSet) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getSchedulerParameterAsProperties";
		// System.out.println(conClassName);
		SchedulerParameters = new HashMap<String, String>();
		try {
			if (isNotNull(pSchedulerParameterSet)) {
				// String strNames = pSchedulerParameterSet.names();
				// logger.debug("Names = " + strNames);
				Set<Map.Entry<String, String>> set = pSchedulerParameterSet.entrySet();
				for (Map.Entry<String, String> entry : set) {
					SchedulerParameters.put(entry.getKey(), entry.getValue());
					SchedulerParameters.put(entry.getKey().replaceAll("_", EMPTY_STRING), entry.getValue());
				}

				set = SchedulerParameters.entrySet();
				for (Map.Entry<String, String> entry : set) {
					String key = entry.getKey();
					String val = entry.getValue();
					String strR = replaceVars(SchedulerParameters, key, val);
					if (strR.equalsIgnoreCase(val) == false) {
						SchedulerParameters.put(key, strR);
					}
				}

			}

			SchedulerParameters = DeleteCurrentNodeNameFromKeys(SchedulerParameters);
			return SchedulerParameters;
		}
		catch (Exception e) {
			throw new JobSchedulerException(JSJ_F_0060.params(e.getMessage()), e);
		}
	}

	public String replaceVars(final HashMap<String, String> params, final String name, String pstrReplaceIn) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::replaceVars";

		if (pstrReplaceIn.matches(".*%[^%]+%.*")) {
			for (String param : params.keySet()) {
				String paramValue = params.get(param);
				if (isNotNull(paramValue)) {
					String paramPattern = "%" + Pattern.quote(param) + "%";
					paramValue = paramValue.replace('\\', '/');
					//	logger.debug(Messages.getMsg(JSJ_D_0031, name, paramPattern, paramValue)); // "processing job parameter '%1$s': substitute '%2$s' with '%3$s'."
					pstrReplaceIn = pstrReplaceIn.replaceAll(paramPattern, paramValue);
				}
				else {
					JSJ_D_0032.toLog(param); // "variable '%1$s' not found. no substitution done"
				}
			}
		}
		return pstrReplaceIn;
	} // private String replaceVars

	/**
	 *
	 * \brief getParameters
	 *
	 * \details
	 * Returns a variable_set with all job and/or order-Parameters.
	 * At the same time, the class-global variable objJobOrderParams is filled.
	 * A Parameter is a   "key and value" tupel.
	 *
	 * \return Variable_set
	 *
	 */
	protected Variable_set getJobOrOrderParameters() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getParameters";
		try {
			Variable_set objJobOrOrderParameters = spooler.create_variable_set();
			objJobOrOrderParameters.merge(getTaskParams());
			if (isJobchain() && hasOrderParameters()) {
				objJobOrOrderParameters.merge(getOrderParams());
			}
			objJobOrOrderParams = objJobOrOrderParameters;
			JSJ_D_0070.toLog(objJobOrOrderParameters.count());
			return objJobOrOrderParameters;
		}
		catch (Exception e) {
			throw new JobSchedulerException(JSJ_F_0050.get(e), e);
		}
	}

	protected Variable_set getTaskParams() {
		return spooler_task.params();
	}

	protected Variable_set getOrderParams() {
		return spooler_task.order().params();
	}

	/**
	 *
	 * \brief getParameters
	 *
	 * \details
	 * Returns a Variable_set with either the job- or the order-parameters.
	 *
	 * \return Variable_set
	 *
	 */
	public Variable_set getParameters() {
		Order order = null;
		try {
			/**
			 * it is important to get a *copy* of the params only instead of a reference to the js-object
			 * if we are working with a reference the params will be in effect until the task comes to an end
			 * all orders/jobs etc which are using the task until it is ended will have the possibly wrong (or too much) parameters
			 */
			Variable_set params = spooler.create_variable_set();
			if (isNotNull(spooler_task.params())) {
				params.merge(spooler_task.params());
			}
			if (isJobchain()) {
				order = spooler_task.order();
				if (isNotNull(order.params())) {
					/**
					 * the order-parameter have higher priority over the task params
					 * sometime a job is using task *and* order-parameter
					 */
					params.merge(order.params());
				}
			}
			objJobOrOrderParams = params;
			return params;
		}
		catch (Exception e) {
			e.printStackTrace();
			String strM = JSJ_F_0010.params(e.getMessage());
			logger.error(strM, e);
			throw new JobSchedulerException(strM, e);
		}
	}

	/**
	 *
	 * \brief getSchedulerParameters
	 *
	 * \details
	 * Returns a Variable_set with the global parameters of scheduler.xml.
	 * Environment variables in the parameter values are already replaced.
	 *
	 * \return Variable_set
	 *
	 */
	public Variable_set getGlobalSchedulerParameters() {
		return spooler.variables();
	}

	/**
	 *
	 * \brief setParameters
	 *
	 * \details
	 * Set all Job- or Order-Parameters as a Variable_set.
	 *
	 * \return void
	 *
	 * @param pVariableSet
	 */
	public void setParameters(final Variable_set pVariableSet) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setParameters";
		objJobOrOrderParams = pVariableSet;
	} // private void setParameters

	/**
	 *
	 * \brief setJSParam
	 *
	 * \details
	 *
	 * @param pstrKey
	 * @param pstrValue
	 * @return
	 */
	@Override
	public void setJSParam(final String pstrKey, final String pstrValue) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setJSParam";
		if (isNotNull(spooler_task.params())) {
			spooler_task.params().set_var(pstrKey, pstrValue);
		}
		if (hasOrderParameters()) {
			spooler_task.order().params().set_var(pstrKey, pstrValue);
		}
		if (isNotNull(objJobOrOrderParams)) {
			objJobOrOrderParams.set_var(pstrKey, pstrValue);
		}
	} //

	/**
	 *
	 * \brief setJSParam
	 *
	 * \details
	 *
	 * \return
	 *
	 * @param pstrKey
	 * @param pstrValue - StringBuffer
	 */
	@Override
	public void setJSParam(final String pstrKey, final StringBuffer pstrValue) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setJSParam";
		setJSParam(pstrKey, pstrValue.toString());
	} // private SOSSSH2SuperClass setJSParam

	/**
	 *
	 * \brief replaceSchedulerVars
	 *
	 * \details
	 *
	 * \return void
	 *
	 * @param isWindows
	 */
	@Override
	public String replaceSchedulerVars(final boolean isWindows, final String pstrString2Modify) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::replaceSchedulerVars";
		String strTemp = pstrString2Modify;
        logger.debug("strTemp = " + strTemp);

		JSJ_D_0080.toLog();

		if (isNotNull(objJobOrOrderParams)) {
			String[] paramNames = objJobOrOrderParams.names().split(";");
			String regExPattern = "(?i)";
			String regex = "(?i)";  // case insensitive
			//
			/**
			 * beides zulassen, % und $
			 * mgliche Kombinationen sind:
			 *
			 * %SCHEDULER_PARAM_name%
			 * %name%
			 * ${SCHEDULER_PARAM_name}
			 * $SCHEDULER_PARAM_name
			 * {SCHEDULER_PARAM_name}
			 * SCHEDULER_PARAM_name
			 * ${name}
			 * {name}
			 * $name
			 * name
			 *
			 * Managed-DB:
			 *    {...}
			 */
			String[] strPatterns = new String[] { "%%SCHEDULER_PARAM_%1$s%%", "%%%1$s%%", "(\\$|)\\{?SCHEDULER_PARAM_%1$s\\}?", "(\\$|)\\{?%1$s\\}?" };
			for (String strPattern : strPatterns) {
				regExPattern = strPattern;
//				logger.debug("regExPattern = " + regExPattern);
				for (String name : paramNames) {
					String strParamValue = objJobOrOrderParams.value(name);
					if (name.contains("password") == false) {
						logger.debug("name = " + name + ", value = " + strParamValue);
					}
					regex = String.format(regExPattern, name);
					strTemp = myReplaceAll(strTemp, regex, strParamValue);
				}
				JSJ_I_0030.toLog( strTemp);
			}
		}
		else {
			JSJ_D_0040.toLog();
		}
		return strTemp;
	}

	public HashMap<String, String> getSpecialParameters() {

		HashMap<String, String> specialParams = new HashMap<String, String>();
		// specialParams.put("SCHEDULER_RETURN_VALUES", remoteCommandScriptOutputParamsFileName);
		specialParams.put("SCHEDULER_HOST", spooler.hostname());
		specialParams.put("SCHEDULER_TCP_PORT", "" + spooler.tcp_port());
		specialParams.put("SCHEDULER_UDP_PORT", "" + spooler.udp_port());
		specialParams.put("SCHEDULER_ID", spooler.id());
		specialParams.put("SCHEDULER_DIRECTORY", spooler.directory());
		specialParams.put("SCHEDULER_CONFIGURATION_DIRECTORY", spooler.configuration_directory());

		if (isJobchain()) {
			specialParams.put("SCHEDULER_JOB_CHAIN", spooler_task.order().job_chain().name());
			specialParams.put("SCHEDULER_JOB_CHAIN_TITLE", spooler_task.order().job_chain().title());
			specialParams.put("SCHEDULER_ORDER_ID", spooler_task.order().id());
			specialParams.put("SCHEDULER_NODE_NAME", getCurrentNodeName());
			specialParams.put("SCHEDULER_NEXT_NODE_NAME", spooler_task.order().job_chain().node(getCurrentNodeName()).next_state());
		}

		specialParams.put("SCHEDULER_JOB_NAME", this.getJobName());
		specialParams.put("SCHEDULER_JOB_TITLE", this.getJobTitle());
		specialParams.put("SCHEDULER_TASK_ID", "" + spooler_task.id());

		try {
			Supervisor_client supervisor = spooler.supervisor_client();
			if (supervisor != null) {
				specialParams.put("SCHEDULER_SUPERVISOR_HOST", supervisor.hostname());
				specialParams.put("SCHEDULER_SUPERVISOR_PORT", "" + supervisor.tcp_port());
			}
		}
		catch (Exception e) {
		}

		return specialParams;
	}

	/**
	 * \brief get all parameters for a task or order
	 * \detail
	 * This method is a combination of getSchedulerParameterAsProperties() and getSpecialParameters()
	 * @return
	 */
	public HashMap<String, String> getAllParametersAsProperties() {
		HashMap<String, String> result = convertVariableSet2HashMap(getGlobalSchedulerParameters());
		result.putAll(getSpecialParameters());
		result.putAll(getSchedulerParameterAsProperties(getParameters()));
		return result;
	}

	/**
	 *
	 * \brief myReplaceAll
	 *
	 * \details
		 * m) suche ber mehrere Zeilen
		 * i) case insensitive
	 *
	 * \return String
	 *
	 * @param source
	 * @param what
	 * @param replacement
	 */
	@Override
	public String myReplaceAll(final String source, final String what, final String replacement) {
		String newReplacement = replacement.replaceAll("\\$", "\\\\\\$");
		/*
		 * m) suche ber mehrere Zeilen
		 * i) case insensitive
		 */
		return source.replaceAll("(?im)" + what, newReplacement);
	}

	/**
	 *
	 * \brief StackTrace2String
	 *
	 * \details
	 * This Method creates a String with all infos from the stack as a trace.
	 *
	 * \return String
	 *
	 * @param e
	 */
	public String StackTrace2String(final Exception e) {
		String strT = null;
		if (isNotNull(e)) {
			strT = e.getMessage() + "\n";
			final StackTraceElement arrStack[] = e.getStackTrace();
			for (final StackTraceElement objS : arrStack) {
				strT += objS.toString() + "\n";
			}
		}
		return strT;
	} // void ShowStackTrace (Exception e)

	protected JSJobUtilities getJSJobUtilities() {
		return this;
	}

	@Override
	public void setJSJobUtilites(final JSJobUtilities pobjJSJobUtilities) {
		// TODO Auto-generated method stub
	}

	/**
	 * moved to JSOptionsClass kb 2011-07-19
	 */
	/**
	 *
	 * \brief getCurrentNodeName
	 *
	 * \details
	 *
	 * \return
	 *
	 * @return
	 */
	// @SuppressWarnings("unchecked")
	// protected HashMap<String, String> DeletePrefix(HashMap<String, String> phsmParameters, String pstrPrefix) {
	//
	// @SuppressWarnings("unused")
	// final String conMethodName = conClassName + "::DeletePrefix";
	// String strTemp;
	// HashMap<String, String> hsmNewMap = new HashMap<String, String>();
	//
	// if (phsmParameters != null) {
	// for (final Object element : phsmParameters.entrySet()) {
	// final Map.Entry<String, String> mapItem = (Map.Entry<String, String>) element;
	// String strMapKey = mapItem.getKey().toString();
	//
	// if (mapItem.getValue() != null) {
	// strTemp = mapItem.getValue().toString();
	// }
	// else {
	// strTemp = null;
	// }
	// // hsmNewMap.put(strMapKey, strTemp);
	// // logger.debug("strMapKey:" + strMapKey + "   strTemp:" +strTemp + "pstrPrefix:" + pstrPrefix );
	// if (strMapKey.startsWith(pstrPrefix)) {
	// // TODO avoid java.util.ConcurrentModificationException
	// // (http://java.sun.com/javase/6/docs/api/java/util/Iterator.html#remove() )
	// // phsmParameters.remove(strMapKey);
	// strMapKey = strMapKey.replaceAll(pstrPrefix, "");
	// // logger.debug("strMapKey after replace:" + strMapKey );
	// hsmNewMap.put(strMapKey, strTemp);
	// mapItem.setValue("\n");
	// }
	// else { // possible case: <nodeName>/<prefix><name> -> <nodeName>/<name>
	// String strP = "/" + pstrPrefix;
	// if (strMapKey.contains(strP)) {
	// strMapKey = strMapKey.replace(strP, "/");
	// hsmNewMap.put(strMapKey, strTemp);
	// mapItem.setValue("\n");
	// }
	// }
	// }
	// }
	//
	// return hsmNewMap;
	// } // private HashMap <String, String> DeletePrefix
	@Override
	// JSJobUtilities
	public String getCurrentNodeName() {
		final String conMethodName = conClassName + "::getNodeName";
		String lstrNodeName = "node1"; // Test, needed for JUnit-Test
		if (spooler_task != null) {
			Order objCurrentOrder = spooler_task.order();
			if (isNotNull(objCurrentOrder)) {
				lstrNodeName = objCurrentOrder.state();
				JSJ_I_0020.toLog(conMethodName, lstrNodeName);
			}
			else {
				Job objCurrentJob = getJob();
				lstrNodeName = objCurrentJob.name();
				JSJ_I_0010.toLog( conMethodName, lstrNodeName);
			}
		}
		return lstrNodeName;
	} // public String getNodeName

	public Job getJob() {
		return spooler_task.job();
	}

	@Override
	public Object getSpoolerObject() {
		return spooler;
	}

	@Override
	public String executeXML(final String pstrJSXmlCommand) {
		String strX = spooler.execute_xml(pstrJSXmlCommand);
		return strX;
	}

	public boolean isOrderJob() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isOrderJob";
		return isJobchain();
	} // private boolean isOrderJob

	public void setNextNodeState(final String pstrNodeName) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setNextNodeState";
		if (isJobchain()) {
			spooler_task.order().set_state(pstrNodeName);
		}
	} // private boolean setNextNodeState

	public boolean isJobchain() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isJobchain";
		boolean flgIsJobChain = false;
		if (isNotNull(spooler_job)) { // && isNotNull(spooler_task)) {
			flgIsJobChain = isNotNull(spooler_job.order_queue()); // || (isNotNull(spooler_task.order()));
		}

		if (flgIsJobChain == false) {
			if (isNotNull(spooler_task)) {
				flgIsJobChain = isNotNull(spooler_task.order());
			}
		}

		// if (flgIsJobChain == true) {
		// logger.info(Messages.getMsg(JSJ_I_0011));
		// }
		// else {
		// logger.info(Messages.getMsg(JSJ_I_0012));
		// }

		return flgIsJobChain;
	} // private boolean isJobchain

	/**
	 *
	 * \brief setOrderParameter
	 *
	 * \details
	 *
	 * \return String
	 *
	 * @param pstrParameterName
	 * @param pstrParameterValue
	 * @return
	 */
	public String setOrderParameter(final String pstrParameterName, final String pstrParameterValue) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setOrderParameter";
		if (isJobchain()) {
			Variable_set objP = getOrderParams();
			if (isNotNull(objP)) {
				objP.set_var(pstrParameterName, pstrParameterValue);
				JSJ_D_0010.toLog(pstrParameterName, pstrParameterValue);
			}
		}
		return pstrParameterValue;
	} // private String setOrderParameter

	public boolean isNotNull(final Object pobjObject) {
		return pobjObject != null;
	}

	public boolean isNull(final Object pobjObject) {
		return pobjObject == null;
	}

	public boolean hasOrderParameters() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::hasOrderParameters";
		boolean flgResult = false;
		if (isJobchain()) {
			flgResult = isNotNull(getOrderParams());
		}
		return flgResult;
	} // private boolean hasOrderParameters
	/**
	 *
	 * 6.3 spooler_process()
	 * The return value is interpreted as with spooler_init().
	 * Should False be returned, then the task will be continued
	 * using spooler_close(); after True the task will be continued with a further call of spooler_process().
	 */
	public final static boolean	conJobSuccess		= false;
	public final static boolean	conJobFailure		= false;
	/**
	 * 6.8.2 Return value of spooler_process()
	 * The return value of order jobs determines if an order was successful or not.
	 *
	 * True: The order will be continued in the next_state of the current job chain node.
	 *
	 * False: The order will be continued in the error_state of the current job chain node.
	 */
	public final static boolean	conJobChainSuccess	= true;
	public final static boolean	conJobChainFailure	= false;

	public boolean signalSuccess() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::signalSuccess";
		if (isJobchain()) {
			return conJobChainSuccess;
		}
		return conJobSuccess;
	} // private boolean signalSuccess

	public boolean signalFailure() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::signalFailure";

		boolean RaiseErrorOnSetback = false;
		String strMsg = JSJ_E_0009.get(this.getJobName());

		if (isJobchain()) {
			if (isSetBackActive() == false || RaiseErrorOnSetback) {
				logger.error(strMsg);
			}
			return conJobChainFailure;
		}
		return conJobFailure;
	} // private boolean signalSuccess

	/*
	 * setBack driven by JS
	 */
	protected boolean isSetBackActive() {
		boolean flgRet = false;

		if (isJobchain()) {
			Order objOrder = spooler_task.order();
			if (objOrder.setback_count() > 0 && objOrder.setback_count() >= getJob().setback_max()) {
				flgRet = true;
			}
		}
		return flgRet;
	}

	private final boolean	flgUseOrderSetBack			= true;
	private final String	conVarname_setback			= "setback";
	private final String	conVarname_setback_count	= "setback_count";

	/**
	 *
	 * \brief setSetbackCount
	 *
	 * \details
	 * SetBack is driven by order parameters.
	 *
	 * \return boolean
	 *
	 * @return
	 */
	//	protected boolean setSetbackCount() {
	//		if (isJobchain()) {
	//			if (sosString.parseToString(schedulerParams.get("use_order_set_back")).length() > 0) {
	//				flgUseOrderSetBack = sosString.parseToBoolean(schedulerParams.get("use_order_set_back"));
	//			}
	//
	//			if (flgUseOrderSetBack) {
	//				int iSetbackCount = spooler_task.order().setback_count();
	//				Variable_set orderParams = spooler_task.order().params();
	//				getLogger().info("setback_count is now: " + iSetbackCount + " , maximum number of setbacks: " + nrOfTries);
	//				if (iSetbackCount >= nrOfTries) {
	//					orderParams.set_var(conVarname_setback_count, "");
	//					getLogger().info("max setbacks reached");
	//				}
	//				else {
	//					spooler_job.set_delay_order_after_setback(1, delay);
	//					/**
	//					 * ist das eigentlich notwendig? der API-Job berwacht doch bereits die Anzahl setBack's
	//					 * Ja, das ist notwendig. im Job werden die max Anzahl setbacks nicht gesetzt, sondern
	//					 * nur ber die Parameter im Order oder Job
	//					 */
	//					spooler_job.set_max_order_setbacks((int) nrOfTries);
	//					spooler_task.order().setback();
	//					iSetbackCount++;
	//					return false;
	//				}
	//			}
	//		}
	//		return true;
	//	}

	/**
	 * @brief Helperfunktion - liefert true, wenn String weder null noch leer
	 *
	 * \details
	 * StringObjekt auf null/empty prfen
	 *
	 * @param pstrValue zu prfendes Stringobjekt
	 *
	 * @return boolean true, wenn String sinnvollen Wert enthlt.
	 */
	protected boolean isNotEmpty(final String pstrValue) {
		return isNotNull(pstrValue) && pstrValue.trim().length() > 0;
	}

	protected boolean isEmpty(final String pstrValue) {
		return isNull(pstrValue) || pstrValue.trim().length() <= 0;
	}

	public Properties mapToProperties(final Map<String, String> map) {
		Properties p = new Properties();
		Set<Map.Entry<String, String>> set = map.entrySet();
		for (Map.Entry<String, String> entry : set) {
			p.put(entry.getKey(), entry.getValue());
		}
		return p;
	}

//	@I18NMessages(value = { @I18NMessage("mandatory jobparameter '%1$s' is not specified"), //
//			@I18NMessage(value = "mandatory jobparameter '%1$s' is not specified", //
//			locale = "en_UK", //
//			explanation = "mandatory jobparameter '%1$s' is not specified" //
//			), //
//			@I18NMessage(value = "Parameter '%1$s' wurde nicht angegeben, ist aber notwendig", locale = "de", //
//			explanation = "jobparameter '%1$s' is missing, but mandatory" //
//			), //
//			@I18NMessage(value = "jobparameter '%1$s' is missing, but mandatory", locale = "es", //
//			explanation = "jobparameter '%1$s' is missing, but mandatory" //
//			), //
//			@I18NMessage(value = "jobparameter '%1$s' is missing, but mandatory", locale = "fr", //
//			explanation = "jobparameter '%1$s' is missing, but mandatory" //
//			), //
//			@I18NMessage(value = "jobparameter '%1$s' is missing, but mandatory", locale = "it", //
//			explanation = "jobparameter '%1$s' is missing, but mandatory" //
//			) //
//	}, msgnum = "JSJ-E-0020", msgurl = "JSJ-E-0020")
///*!
// * \var MsgKey
// * \brief jobparameter '%1$s' is missing, but mandatory
// */
//	public static final String	JSJ_E_0020	= "JSJ_E_0020";
//	//
//	@I18NMessages(value = { @I18NMessage("error occurred in class %1$s : %2$s"), //
//			@I18NMessage(value = "error occurred in %1$s : %2$s", //
//			locale = "en_UK", //
//			explanation = "error occurred in class %1$s : %2$s" //
//			), //
//			@I18NMessage(value = "Fehler in der Klasse %1$s aufgetreten: %2$s", //
//			locale = "de", //
//			explanation = "Ein fataler Fehler ist in der Klasse %1$s aufgetreten. Meldung: %2$s" //
//			), //
//			@I18NMessage(value = "error occurred in %1$s : %2$s", locale = "es", //
//			explanation = "error occurred in %1$s : %2$s" //
//			), //
//			@I18NMessage(value = "error occurred in %1$s : %2$s", locale = "fr", //
//			explanation = "error occurred in %1$s : %2$s" //
//			), //
//			@I18NMessage(value = "error occurred in %1$s : %2$s", locale = "it", //
//			explanation = "error occurred in %1$s : %2$s" //
//			) //
//	}, msgnum = "JSJ-F-0010", msgurl = "JSJ-F-0010")
///*!
// * \var JSJ_F_0010
// * \brief error occurred in %1$s : %2$s
// */
//	public static final String	JSJ_F_0010	= "JSJ_F_0010";
//	@I18NMessages(value = { @I18NMessage(".. get jobparameter '%1$s' = '%2$s'."), //
//			@I18NMessage(value = ".. get jobparameter '%1$s' = '%2$s'.", //
//			locale = "en_UK", //
//			explanation = ".. get jobparameter '%1$s' = '%2$s'." //
//			), //
//			@I18NMessage(value = ".. lese Parameter '%1$s' mit Wert '%2$s'.", //
//			locale = "de", //
//			explanation = ".. get jobparameter '%1$s' = '%2$s'." //
//			), //
//			@I18NMessage(value = ".. get jobparameter '%1$s' = '%2$s'.", locale = "es", //
//			explanation = ".. get jobparameter '%1$s' = '%2$s'." //
//			), //
//			@I18NMessage(value = ".. get jobparameter '%1$s' = '%2$s'.", locale = "fr", //
//			explanation = ".. get jobparameter '%1$s' = '%2$s'." //
//			), //
//			@I18NMessage(value = ".. get jobparameter '%1$s' = '%2$s'.", locale = "it", //
//			explanation = ".. get jobparameter '%1$s' = '%2$s'." //
//			) //
//	}, msgnum = "JSJ-E-0030", msgurl = "JSJ-E-0030")
//	/*!
//	 * \var JSJ_E_0030
//	 * \brief .. get jobparameter '%1$s' = '%2$s'.
//	 */
//	public static final String	JSJ_I_0040	= "JSJ_I_0040";
//	@I18NMessages(value = { @I18NMessage("No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'."), //
//			@I18NMessage(value = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'.", //
//			locale = "en_UK", //
//			explanation = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'." //
//			), //
//			@I18NMessage(value = "Anzahl Treffer '$1$d' in der Ergebnisliste ist '%2$s', erwartet wird '$3$d'. Fehler.", //
//			locale = "de", //
//			explanation = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'." //
//			), //
//			@I18NMessage(value = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'.", locale = "es", //
//			explanation = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'." //
//			), //
//			@I18NMessage(value = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'.", locale = "fr", //
//			explanation = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'." //
//			), //
//			@I18NMessage(value = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'.", locale = "it", //
//			explanation = "No of Hits in Result Set '$1$d'  is '%2$s' expected '$3$d'." //
//			) //
//	}, msgnum = "JSJ_E_0040", msgurl = "JSJ_E_0040")
//	/*!
//	 * \var JSJ_E_0040
//	 * \brief No of Hits in Result Set '$1$d'  is '%2$s' expected '$1$d'.
//	 */
//	public static final String	JSJ_E_0040	= "JSJ_E_0040";

	// private String LOG_I_0010 = "Log4j configured programmatically";
	// private String LOG_I_0020 = "JobSchedulerLog4JAppender is configured as log4j-appender";
	//
	// private String JSJ_I_0010 = "%1$s: Current Job is '%2$s'.";
	// private String JSJ_I_0011 = "Job is running in a job-chain";
	// private String JSJ_I_0012 = "Job is running as a standalone-job";
	// public String JSJ_I_0017 = "Order created by %1$s";
	// public String JSJ_I_0018 = "Order '%1$s' created for JobChain '%2$s'.";
	// public String JSJ_I_0019 = "Next State is '%1$s'.";
	// private String JSJ_I_0020 = "%1$s: Current NodeName is '%2$s'.";
	// private String JSJ_I_0030 = "string after replacing-operation is = %1$s";
	// public String JSJ_I_0090 = "set order-state to '%1$s'";
	//
	// public String JSJ_F_0015 = "error occurred instantiating '%1$s': %2$s";
	// public String JSJ_F_0016 = "error occurred in '%1$s': %2$s";
	// public String JSJ_F_0050 = "error occurred getting Parameters for job or order: %1$s";
	// public String JSJ_F_0060 = "error occurred reading Parameter from Variable_set : '%1$s'";
	// public static String JSJ_F_0080 = "Failed to write '%2$s' to '%1$s'";
	// public static final String JSJ_F_0090 = "File '%1$s' for '%2$s' is not writable";
	//
	// private String JSJ_D_0010 = "set parameter '%1$s' to value '%2$s'";
	// public String JSJ_D_0031 = "processing job parameter '%1$s': substitute '%2$s' with '%3$s'.";
	// public String JSJ_D_0032 = "variable '%1$s' not found. no substitution done";
	// private String JSJ_D_0040 = "No job- or order-parameters found. No replacing was done.";
	// public String JSJ_D_0044 = "processing job parameter '%1$s': substitute '%2$s' with '%3$s'.";
	// private String JSJ_D_0070 = "Number of Parameters found for Job or Order is : %1$d";
	// private String JSJ_D_0080 = "Start replacing of job- and/or order-parameters .";
	//
	// public String JSJ_E_0017 = "Compare operator not known: '%1$s'";
	// public String JSJ_E_0041 = "Order '%1$s' not found";
	// public String JSJ_E_0042 = "exception raised %1$s";
	//
	// public static String JSJ_E_0110 = "parameter '%1$s' is missing but required for parameter '%2$s'";
	// public String JSJ_E_0120 =
	// "This job runs not in a job chain (order driven job). Job parameter '%1$s' is only valid for order driven jobs";
	// public String JSJ_E_0130 = "invalid, non-numeric value for parameter '%1$s': %2$s";
	//
	// public String JSJ_W_0043 = "variable '%1$s' not found. no substitution done";
	//
	// public String JSJ_T_0010 = "greater or equal";

//	private final String		LOG_I_0010	= "LOG_I_0010"; // Log4j configured programmatically
//	private final String		LOG_I_0020	= "LOG_I_0020"; // JobSchedulerLog4JAppender is configured as log4j-appender
//
//	private final String		JSJ_E_0009	= "JSJ_E_0009"; // "Job '%1$s' terminated with error".
//	private final String		JSJ_I_0010	= "JSJ_I_0010"; // %1$s: Current Job is '%2$s'.
//	private final String		JSJ_I_0011	= "JSJ_I_0011"; // Job is running in a job-chain
//	private final String		JSJ_I_0012	= "JSJ_I_0012"; // Job is running as a standalone-job
//	public String				JSJ_I_0017	= "JSJ_I_0017"; // Order created by %1$s
//	public String				JSJ_I_0018	= "JSJ_I_0018"; // Order '%1$s' created for JobChain '%2$s'.
//	public String				JSJ_I_0019	= "JSJ_I_0019"; // Next State is '%1$s'.
//	private final String		JSJ_I_0020	= "JSJ_I_0020"; // %1$s: Current NodeName is '%2$s'.
//	private final String		JSJ_I_0030	= "JSJ_I_0030"; // string after replacing-operation is = %1$s
//	public String				JSJ_I_0090	= "JSJ_I_0090"; // set order-state to '%1$s'
//
////	public String				JSJ_F_0015	= "JSJ_F_0015"; // error occurred instantiating '%1$s': %2$s
////	public String				JSJ_F_0016	= "JSJ_F_0016"; // error occurred in '%1$s': %2$s
//	public String				JSJ_F_0050	= "JSJ_F_0050"; // error occurred getting Parameters for job or order: %1$s
//	public String				JSJ_F_0060	= "JSJ_F_0060"; // error occurred reading Parameter from Variable_set : '%1$s'
//	public static String		JSJ_F_0080	= "JSJ_F_0080"; // Failed to write '%2$s' to '%1$s'
//	public static final String	JSJ_F_0090	= "JSJ_F_0090"; // File '%1$s' for '%2$s' is not writable
//
//	private final String		JSJ_D_0010	= "JSJ_D_0010"; // set parameter '%1$s' to value '%2$s'
//	public String				JSJ_D_0031	= "JSJ_D_0031"; // processing job parameter '%1$s': substitute '%2$s' with '%3$s'.
//	public String				JSJ_D_0032	= "JSJ_D_0032"; // variable '%1$s' not found. no substitution done
//	private final String		JSJ_D_0040	= "JSJ_D_0040"; // No job- or order-parameters found. No replacing was done.
//	public String				JSJ_D_0044	= "JSJ_D_0044"; // processing job parameter '%1$s': substitute '%2$s' with '%3$s'.
//	private final String		JSJ_D_0070	= "JSJ_D_0070"; // Number of Parameters found for Job or Order is : %1$d
//	private final String		JSJ_D_0080	= "JSJ_D_0080"; // Start replacing of job- and/or order-parameters .
//
//	public String				JSJ_E_0017	= "JSJ_E_0017"; // Compare operator not known: '%1$s'
////	public String				JSJ_E_0041	= "JSJ_E_0041"; // Order '%1$s' not found
//	public String				JSJ_E_0042	= "JSJ_E_0042"; // exception raised %1$s
//
//	public static String		JSJ_E_0110	= "JSJ_E_0110"; // parameter '%1$s' is missing but required for parameter '%2$s'
//	public String				JSJ_E_0120	= "JSJ_E_0120"; // This job runs not in a job chain (order driven job). Job parameter '%1$s' is
//															// only valid for order driven jobs
//	public String				JSJ_E_0130	= "JSJ_E_0130"; // invalid, non-numeric value for parameter '%1$s': %2$s
//
//	public String				JSJ_W_0043	= "JSJ_W_0043"; // variable '%1$s' not found. no substitution done
//
//	public String				JSJ_T_0010	= "JSJ_T_0010"; // greater or equal
//
	@Override
	// IJobSchedulerMonitor_impl
	public boolean spooler_task_before() throws Exception {
		final String conMethodName = conClassName + "::spooler_task_before";
		initializeLog4jAppenderClass();
		boolean ret = continueWithProcessBefore;
		//boolean ret = continueWithTaskAfter;
		logger.info(String.format("%1$s is running and returns %2$s", conMethodName, ret));
		return ret;
	}

	@Override
	// IJobSchedulerMonitor_impl
	public void spooler_task_after() throws Exception {
		final String conMethodName = conClassName + "::spooler_task_after";
		initializeLog4jAppenderClass();
		logger.info(String.format("%1$s is running", conMethodName));
	}

	@Override
	// IJobSchedulerMonitor_impl
	public boolean spooler_process_before() throws Exception {
		final String conMethodName = conClassName + "::spooler_process_before";
		initializeLog4jAppenderClass();
		boolean ret = continueWithProcess;
		//boolean ret = continueWithTaskAfter;
		logger.info(String.format("%1$s is running and returns %2$s", conMethodName, ret));
		return ret;
	}

	@Override
	// IJobSchedulerMonitor_impl
	public boolean spooler_process_after(final boolean spooler_process_result) throws Exception {
		final String conMethodName = conClassName + "::spooler_process_after";
		initializeLog4jAppenderClass();
		logger.info(String.format("%1$s is running and returns %2$s", conMethodName, spooler_process_result));
		return spooler_process_result;
	}

	@Override
	public void setStateText(final String pstrStateText) {
		if (isJobchain()) {
			spooler_task.order().set_state_text(pstrStateText);
		}
	}
}
